@@ -3,70 +3,47 @@ from .nn.optimizers import get_optimizer
 from .utils.toolkit import *
 from .nn.core import Variable, Layer
 from .nn.functional import concatenate
-from .nn.global_graph import np, topological_sort
+from .nn import global_graph as GlobalGraph
 import time
 import pickle
 from typing import Tuple
 
 
-class Module:
-    def __init__(self, *args):
-        self.trainable_variables = []  # Variable数组
+class Base:
+    def __init__(self):
+        self.variables = set()
+
+    def train(self):
+        GlobalGraph.IS_TRAINING = True
+
+    def eval(self):
+        GlobalGraph.IS_TRAINING = False
+
+
+class _Model(Base):
+    def __init__(self):
+        super().__init__()
         self.loss = None  # Variable
         self.optimizer = None  # Optimizer类型
         self.graph = None  # Layer数组
-        self.is_training = True  # 布尔
-
-    def __call__(self, x: Variable, *args, **kwargs):
-        out = self.forward(x)
-        out.retain_grad()
-        self.__add_trainable_variables(x)
-        return out
-
-    def __add_trainable_variables(self, x: Variable):
-        # 用一个列表来模拟队列，然后bfs遍历统计所有的trainable_variables
-        queue = [x]
-        seen = set()  # 此处为set, python里set用的是hash table, 搜索时比数组要快。
-        seen.add(x)
-        while queue:
-            # 队列非空，队首元素出列
-            vertex = queue.pop(0)
-            # 将该元素的邻接元素加入队尾
-            for n in vertex.out_bounds:
-                if n not in seen:
-                    for v in n.in_bounds:
-                        if v is not None and v.requires_grad and v.name == 'xs_variable' and v not in self.trainable_variables:
-                            self.trainable_variables.append(v)
-
-                    queue.append(n)
-                    seen.add(n)
-
-    def parameters(self):
-        return self.trainable_variables
 
     def compile(self, optimizer, loss):
         raise NotImplemented
 
-    def train(self):
-        self.is_training = True
-
-    def eval(self):
-        self.is_training = False
-
     def fit(self,
-            x: np.ndarray,
-            y: np.ndarray,
+            x: GlobalGraph.np.ndarray,
+            y: GlobalGraph.np.ndarray,
             batch_size: int = None,
             epochs: int = 1,
             verbose: int = 1,
             shuffle: bool = True,
-            validation_data: Tuple[np.ndarray] = None,
+            validation_data: Tuple[GlobalGraph.np.ndarray] = None,
             validation_split: float = 0.,
             initial_epoch: int = 0,
             ) -> dict:
 
-        x = np.asarray(x)
-        y = np.asarray(y)
+        x = GlobalGraph.np.asarray(x)
+        y = GlobalGraph.np.asarray(y)
         if validation_data is None and 0. < validation_split < 1.:
             split = int(x.shape[0] * validation_split)
             valid_x, valid_y = x[-split:], y[-split:]
@@ -100,13 +77,11 @@ class Module:
             progress_bar = ProgressBar(max_iter=len(train_x), verbose=verbose)
 
             for idx, (xs, ys) in enumerate(train_dataloader):
-                xs = Variable(xs)
-                ys = Variable(ys)
                 self.train()
                 # reset trainable_variables grad
                 self.optimizer.zero_grad()
                 # forward
-                pred = self.forward(xs)
+                pred = self.forward(xs, is_training=True)
                 loss = self.loss.forward(pred, ys)
                 self.backward(loss)
                 self.optimizer.step()
@@ -128,10 +103,22 @@ class Module:
             print()
         return history
 
-    def evaluate(self, x: np.ndarray, y: np.ndarray, batch_size: int = None):
-        x = np.asarray(x)
-        y = np.asarray(y)
+    def __call__(self, x: Variable, *args, **kwargs):
+        out = self.forward(x)
+        return out
+
+    def forward(self, x, *args, **kwargs):
+        raise NotImplemented
+
+    def backward(self, loss: Variable):
+        loss.grad_fn(loss)
+        for layer in reversed(self.graph):
+            layer.backward()
+
+    def evaluate(self, x: GlobalGraph.np.ndarray, y: GlobalGraph.np.ndarray, batch_size: int = None):
         self.eval()
+        x = GlobalGraph.np.asarray(x)
+        y = GlobalGraph.np.asarray(y)
         if batch_size is not None:
             assert type(batch_size) is int
             val_dataset = DataSet(x, y)
@@ -139,24 +126,20 @@ class Module:
             acc_list = []
             loss_list = []
             for xs, ys in val_dataloader:
-                xs = Variable(xs)
-                ys = Variable(ys)
                 y_pred = self.forward(xs)
                 metric = self.loss.metric(y_pred, ys)
                 acc_list.append(metric[0])
                 loss_list.append(metric[1])
 
-            acc = np.array(acc_list).mean().tolist()
-            loss = np.array(loss_list).mean().tolist()
+            acc = GlobalGraph.np.array(acc_list).mean().tolist()
+            loss = GlobalGraph.np.array(loss_list).mean().tolist()
         else:
-            x = Variable(x)
-            y = Variable(y)
-            y_pred = self.forward(x)
+            y_pred = self.forward(Variable(x))
             acc, loss = self.loss.metric(y_pred, y)
 
         return acc, loss
 
-    def predict(self, x: np.ndarray, batch_size: int = None):
+    def predict(self, x: GlobalGraph.np.ndarray, batch_size: int = None):
         self.eval()
         if batch_size is not None:
             assert type(batch_size) is int
@@ -164,23 +147,14 @@ class Module:
             test_dataloader = DataLoader(test_dataset, batch_size)
             pred_list = []
             for xs in test_dataloader:
-                xs = Variable(xs)
                 y_pred = self.forward(xs)
                 pred_list.append(y_pred)
             pred = concatenate(*pred_list, axis=0)
         else:
-            pred = self.forward(x)
+            pred = self.forward(Variable(x))
 
         return pred
 
-    def forward(self, x):
-        raise NotImplemented
-
-    def backward(self, loss: Variable):
-        loss.grad_fn(loss)
-        for layer in reversed(self.graph):
-            layer.backward()
-
     def save(self, save_path):
         if not save_path.endswith('.pkl'):
             save_path += '.pkl'
@@ -202,13 +176,11 @@ class Module:
         print('*' * bar_nums)
         print('Layer(type)'.ljust(25), 'Output Shape'.ljust(20), 'Param'.ljust(10), 'Connected to'.ljust(15))
         print('#' * bar_nums)
-        total_params = 0
         if self.graph is None:
             raise ValueError('Please compile Model!')
         for layer in self.graph:
             layer_name = '%s (%s)' % (layer.name, layer.__class__.__name__)
             params = layer.params_count()
-            total_params += params
             first = True
             if layer.in_bounds:
                 for prev_layer in layer.in_bounds:
@@ -229,16 +201,19 @@ class Module:
             print('-' * bar_nums)
 
         print('*' * bar_nums)
+        total_params = 0
         trainable_params = 0
-        for v in self.trainable_variables:
-            trainable_params += v.data.size
+        for v in self.variables:
+            total_params += v.data.size
+            if v.requires_grad:
+                trainable_params += v.data.size
         params_details = 'Total params: %d\n' % total_params
         params_details += 'Trainable params: %d\n' % trainable_params
         params_details += 'Non-trainable params: %d\n' % (total_params - trainable_params)
         return params_details
 
 
-class Sequential(Module):
+class Sequential(_Model):
     def __init__(self, *layers: Layer):
         super().__init__()
         self.graph = [] if layers is None else list(layers)
@@ -254,17 +229,16 @@ class Sequential(Module):
             layer.initial_params()
             next_layer = layer
             for v in layer.variables:
-                if v is not None and v.requires_grad and v not in self.trainable_variables:
-                    self.trainable_variables.append(v)
+                if v is not None:
+                    self.variables.add(v)
         self.loss = get_objective(loss)
-        self.optimizer = get_optimizer(optimizer, **kwargs)
-        self.optimizer.trainable_variables = self.trainable_variables
+        self.optimizer = get_optimizer(optimizer, variables=self.variables, **kwargs)
 
-    def forward(self, x):
+    def forward(self, x, *args, **kwargs):
         for layer in self.graph:
-            if hasattr(layer, 'variables') and len(layer.variables) == 0:
+            if isinstance(layer, Layer) and len(layer.variables) == 0:
                 layer.initial_params(x.shape[1:])
-            x = layer.forward(x, self.is_training)
+            x = layer.forward(x)
         return x
 
     def pop(self, index=-1):
@@ -272,20 +246,8 @@ class Sequential(Module):
         del layer
         print('success delete %s layer' % layer.__class__.__name__)
 
-    def save(self, save_path):
-        with open(save_path + '.pkl', 'wb') as f:
-            pickle.dump([self.graph, self.optimizer, self.loss], f)
 
-    def load(self, model_path):
-        with open(model_path + '.pkl', 'rb') as f:
-            layers, optimizer, loss = pickle.load(f)
-
-        self.graph = layers
-        self.optimizer = optimizer
-        self.loss = loss
-
-
-class Model(Module):
+class Model(_Model):
     def __init__(self, inputs=None, outputs=None):
         super().__init__()
         self.inputs = inputs
@@ -293,19 +255,52 @@ class Model(Module):
 
     def compile(self, optimizer, loss, **kwargs):
         assert self.inputs is not None and self.outputs is not None
-        self.graph = topological_sort(self.inputs, self.outputs)
+        self.graph = GlobalGraph.topological_sort(self.inputs, self.outputs)
         for g in self.graph:
             g.initial_params()
             for v in g.variables:
-                if v is not None and v.requires_grad and v not in self.trainable_variables:
-                    self.trainable_variables.append(v)
+                if v is not None:
+                    self.variables.add(v)
         self.loss = get_objective(loss)
-        self.optimizer = get_optimizer(optimizer, **kwargs)
-        self.optimizer.trainable_variables = self.trainable_variables
+        self.optimizer = get_optimizer(optimizer, variables=self.variables, **kwargs)
+
+    def forward(self, x: Variable, *args, **kwargs):
 
-    def forward(self, x: Variable):
         self.inputs.input_data = x
         outputs = None
         for layer in self.graph:
-            outputs = layer.forward(is_training=self.is_training)
+            outputs = layer.forward()
         return outputs
+
+
+class Module(Base):
+    def __init__(self, *args):
+        super().__init__()
+
+    def __call__(self, x: Variable, *args, **kwargs):
+        out = self.forward(x)
+        self.__collect_variables(x)
+        return out
+
+    def __collect_variables(self, x: Variable):
+        # 用一个列表来模拟队列，然后bfs遍历统计所有的trainable_variables
+        queue = [x]
+        seen = set()  # 此处为set, python里set用的是hash table, 搜索时比数组要快。
+        seen.add(x)
+        while queue:
+            # 队列非空，队首元素出列
+            vertex = queue.pop(0)
+            # 将该元素的邻接元素加入队尾
+            for n in vertex.out_bounds:
+                if n not in seen:
+                    for v in n.get_variables():
+                        if v is not None:
+                            self.variables.add(v)
+                    queue.append(n)
+                    seen.add(n)
+
+    def parameters(self):
+        return self.variables
+
+    def forward(self, x):
+        raise NotImplemented
