@@ -17,9 +17,6 @@ class Node:
         :param name: tensor的名字，字符串
         :param requires_grad: 需不需要梯度， 布尔值
         """
-
-        # 下面第一行这句话的意思是如果传递了in_bounds参数，那就把传递的in_bounds先包装为数组，在赋给类成员in_bounds，否则的话初始化类成员in_bounds为空数组，如果c++写的话大概是这样：
-        # this->in_bounds = in_bounds == NULL ?  vector<Template> : vector<Template>(in_bounds)
         self.in_bounds = [] if in_bounds is None else list(in_bounds)
         self.out_bounds = [] if out_bounds is None else list(out_bounds)
         self.data = data
@@ -36,6 +33,14 @@ class Node:
         self.cache = {}
         # bp后是否需要保存梯度
         self.retain = False
+        # 计算得到该Node所需要的参数，比如weight, bias
+        self.__parameters = []
+
+    def get_variables(self):
+        return self.__parameters
+
+    def set_variables(self, variables: List):
+        self.__parameters = variables
 
     def retain_grad(self):
         self.retain = True
@@ -70,14 +75,14 @@ class Node:
     # 加法
     def __add__(self, other):
         # 全局运算图
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
         return add(self, other)
 
     # 加等于
     def __iadd__(self, other):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
 
         self.data += other.data
         self.in_bounds.append(other)
@@ -85,32 +90,42 @@ class Node:
         if 'grad_fn' not in self.cache:
             self.cache['grad_fn'] = []
         self.cache['grad_fn'].append(self.grad_fn)
-        self.grad_fn = IAddBackward
+        if GlobalGraph.IS_TRAINING and self.requires_grad:
+            self.grad_fn = IAddBackward
+            initialize_ops_grad(self, other)
         return self
 
+    def __eq__(self, other):
+        if self.data.size != other.data.size:
+            return False
+        return Variable(self.data == other.data, requires_grad=self.requires_grad or other.requires_grad)
+
+    def __hash__(self):
+        return hash(id(self))
+
     # 取负号
     def __neg__(self):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
         return neg(self)
 
     # 减法
     def __sub__(self, other):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
         return sub(self, other)
 
     # 乘法
     def __mul__(self, other):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
         return mul(self, other)
 
     # 幂
     def __pow__(self, power: int, modulo=None):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
-        outputs = Variable(in_bounds=[self, ], data=np.power(self.data, power), requires_grad=True)
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
+        outputs = Variable(in_bounds=[self, ], data=GlobalGraph.np.power(self.data, power), requires_grad=True)
         # 绑定反向求梯度的函数
         outputs.grad_fn = PowBackward
         outputs.cache['power'] = power
@@ -132,17 +147,17 @@ class Node:
             # 如果是标量，默认该标量梯度为1
             if self.data.size == 1:
                 if self.grad is None:
-                    self.grad = np.array(1.)
+                    self.grad = GlobalGraph.np.array(1.)
             else:
                 raise ValueError('grad can be implicitly created only for scalar outputs')
 
         # self.grad_fn(self)
-        if GlobalGraph.outputs is None:
-            GlobalGraph.outputs = self
-        if GlobalGraph.inputs is not None:
+        if GlobalGraph.OUTPUTS is None:
+            GlobalGraph.OUTPUTS = self
+        if GlobalGraph.INPUTS is not None:
             graph = GlobalGraph.build_graph()
-            GlobalGraph.inputs.retain_grad()
-            GlobalGraph.outputs.retain_grad()
+            GlobalGraph.INPUTS.retain_grad()
+            GlobalGraph.OUTPUTS.retain_grad()
             for node in reversed(graph):
                 if node.grad_fn is not None:
                     node.grad_fn(node)
@@ -158,22 +173,22 @@ class Node:
         return self.data.shape if axis is None else self.data.shape[axis]
 
     def zero_grad(self):
-        self.grad = np.zeros_like(self.data)
+        self.grad = GlobalGraph.np.zeros_like(self.data)
 
     # 把数据转换为长整型，返回一个Variable
     def long(self):
-        output = Variable(data=self.data, dtype=np.int64, requires_grad=self.requires_grad)
+        output = Variable(data=self.data, dtype=GlobalGraph.np.int64, requires_grad=self.requires_grad)
         return output
 
     # 方法名后面加一个_指在本身数据上直接修改，没有返回值
     def long_(self):
-        self.data = self.data.astype(np.int64)
+        self.data = self.data.astype(GlobalGraph.np.int64)
 
     # 矩阵乘
     def matmul(self, other):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
-        outputs_data = np.dot(self.data, other.data)
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
+        outputs_data = GlobalGraph.np.dot(self.data, other.data)
         outputs = Variable(data=outputs_data, in_bounds=[self, other], requires_grad=self.requires_grad or other.requires_grad)
         if outputs.requires_grad:
             outputs.grad_fn = MatmulBackward
@@ -187,8 +202,8 @@ class Node:
 
     # 转置
     def t(self):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
         outputs = Variable(data=self.data.T, in_bounds=[self, ], requires_grad=self.requires_grad)
         if outputs.requires_grad:
             outputs.grad_fn = TransposeBackward
@@ -198,13 +213,13 @@ class Node:
 
     # 求和
     def sum(self, axis: int = None, keepdims: bool = False):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
         # 在哪个axis（维度）上求和
         if axis is None:
-            sum_value = np.sum(self.data, keepdims=keepdims)
+            sum_value = GlobalGraph.np.sum(self.data, keepdims=keepdims)
         else:
-            sum_value = np.sum(self.data, axis=axis, keepdims=keepdims)
+            sum_value = GlobalGraph.np.sum(self.data, axis=axis, keepdims=keepdims)
         outputs = Variable(in_bounds=[self, ], data=sum_value, requires_grad=self.requires_grad)
         if outputs.requires_grad:
             outputs.cache['axis'] = axis
@@ -216,13 +231,13 @@ class Node:
 
     # 求均值
     def mean(self, axis: int = None, keepdims: bool = False):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
 
         if axis is None:
-            mean_value = np.mean(self.data, keepdims=keepdims)
+            mean_value = GlobalGraph.np.mean(self.data, keepdims=keepdims)
         else:
-            mean_value = np.mean(self.data, axis=axis, keepdims=keepdims)
+            mean_value = GlobalGraph.np.mean(self.data, axis=axis, keepdims=keepdims)
         outputs = Variable(in_bounds=[self, ], data=mean_value, requires_grad=self.requires_grad)
         if outputs.requires_grad:
             outputs.cache['axis'] = axis
@@ -234,10 +249,10 @@ class Node:
 
     # 求绝对值
     def abs(self):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
 
-        outputs = Variable(in_bounds=[self, ], data=np.abs(self.data), requires_grad=self.requires_grad)
+        outputs = Variable(in_bounds=[self, ], data=GlobalGraph.np.abs(self.data), requires_grad=self.requires_grad)
         if outputs.requires_grad:
             # 绑定反向求梯度的函数
             outputs.grad_fn = AbsBackward
@@ -247,8 +262,8 @@ class Node:
 
     # reshape，传入的参数应该是一个list或者tuple
     def view(self, *shapes):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
 
         outputs = Variable(in_bounds=[self, ], data=self.data.reshape(*shapes), requires_grad=self.requires_grad)
         if outputs.requires_grad:
@@ -260,16 +275,16 @@ class Node:
 
     # 求log
     def log(self, base: int = 2):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
 
         # 这里可能需要重载一下, base可选为2， 10和'e'
         if base == 2:
-            ret_value = np.log2(self.data)
+            ret_value = GlobalGraph.np.log2(self.data)
         elif base == 10:
-            ret_value = np.log10(self.data)
+            ret_value = GlobalGraph.np.log10(self.data)
         else:
-            ret_value = np.log(self.data)
+            ret_value = GlobalGraph.np.log(self.data)
         outputs = Variable(in_bounds=[self, ], data=ret_value, requires_grad=self.requires_grad)
         if outputs.requires_grad:
             # 绑定反向求梯度的函数
@@ -282,10 +297,10 @@ class Node:
 
     # 求exp
     def exp(self):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
 
-        outputs = Variable(in_bounds=[self, ], data=np.exp(self.data), requires_grad=self.requires_grad)
+        outputs = Variable(in_bounds=[self, ], data=GlobalGraph.np.exp(self.data), requires_grad=self.requires_grad)
         if outputs.requires_grad:
             # 绑定反向求梯度的函数
             outputs.grad_fn = ExpBackward
@@ -293,10 +308,13 @@ class Node:
             self.out_bounds.append(outputs)
         return outputs
 
-    def max(self, axis=None):
-        if GlobalGraph.inputs is None:
-            GlobalGraph.inputs = self
-        outputs = Variable(in_bounds=[self, ], data=np.max(self.data, axis=axis), requires_grad=self.requires_grad)
+    def max(self, axis=None, **kwargs):
+        if GlobalGraph.INPUTS is None:
+            GlobalGraph.INPUTS = self
+        requires_grad = kwargs.pop('requires_grad', False)
+        keepdims = kwargs.pop('keepdims', False)
+        outputs = Variable(in_bounds=[self, ], data=GlobalGraph.np.max(self.data, axis=axis, keepdims=keepdims),
+                           requires_grad=self.requires_grad or requires_grad, **kwargs)
         if outputs.requires_grad:
             outputs.cache['axis'] = axis
             outputs.grad_fn = MaxBackward
@@ -304,8 +322,9 @@ class Node:
             self.out_bounds.append(outputs)
         return outputs
 
-    def argmax(self, axis=None):
-        outputs = Variable(in_bounds=[self, ], data=np.argmax(self.data, axis=axis), requires_grad=self.requires_grad)
+    def argmax(self, axis=None, **kwargs):
+        requires_grad = kwargs.pop('requires_grad', False)
+        outputs = Variable(in_bounds=[self, ], data=GlobalGraph.np.argmax(self.data, axis=axis), requires_grad=self.requires_grad or requires_grad)
         return outputs
 
     def numel(self):
@@ -315,18 +334,27 @@ class Node:
 class Variable(Node):
     def __init__(self, data: Union[GlobalGraph.np.ndarray, int, float], in_bounds: List = None,
                  out_bounds: Union[List, Tuple] = None,
-                 name: str = None, requires_grad: bool = True, dtype: str = 'float64'):
-        # Variable初始化时必须提供data值，data值可能传入的是一个int或者float，我们需要把它包装成numpy矩阵（cpp中就包装成我们选的矩阵库类型）
-        dtype_dict = {'int': np.int, 'float': np.float, 'int8': np.int8, 'int16': np.int16, 'int32': np.int32,
-                      'int64': np.int64, 'float32': np.float32, 'float64': np.float64}
-        data = np.asarray(data, dtype=dtype_dict[dtype])  # 数据类型用float64吧，float32也行
-        Node.__init__(self,
-                      in_bounds=in_bounds,
-                      out_bounds=out_bounds,
-                      data=data,
-                      shape=data.shape,
-                      name=name,
-                      requires_grad=requires_grad)
+                 name: str = None, requires_grad: bool = True, dtype: str = 'float32'):
+        if isinstance(data, Variable):
+            Node.__init__(self,
+                          in_bounds=data.in_bounds,
+                          out_bounds=data.out_bounds,
+                          data=data.data,
+                          shape=data.data.shape,
+                          name=data.name,
+                          requires_grad=data.requires_grad)
+        else:
+            # Variable初始化时必须提供data值，data值可能传入的是一个int或者float，我们需要把它包装成numpy矩阵（cpp中就包装成我们选的矩阵库类型）
+            dtype_dict = {'int': GlobalGraph.np.int, 'float': GlobalGraph.np.float, 'int8': GlobalGraph.np.int8, 'int16': GlobalGraph.np.int16, 'int32': GlobalGraph.np.int32,
+                          'int64': GlobalGraph.np.int64, 'float32': GlobalGraph.np.float32, 'float64': GlobalGraph.np.float64}
+            data = GlobalGraph.np.asarray(data, dtype=dtype_dict[dtype])  # 数据类型用float64吧，float32也行
+            Node.__init__(self,
+                          in_bounds=in_bounds,
+                          out_bounds=out_bounds,
+                          data=data,
+                          shape=data.shape,
+                          name=name,
+                          requires_grad=requires_grad)
 
     def __getitem__(self, item):
         return slices(self, item)
@@ -338,12 +366,12 @@ class Variable(Node):
 class Constant(Node):
     def __init__(self, data: Union[GlobalGraph.np.ndarray, int, float], in_bounds: List = None,
                  out_bounds: Union[List, Tuple] = None,
-                 name: str = None, dtype: str = 'float64'):
+                 name: str = None, dtype: str = 'float32'):
         # Constant初始化时必须提供data值，并且一旦初始化就不可修改，Constant因为值不需要修改，也就没必要计算梯度，默认require_grads为False
         # !!!!!!!!!!!!!!!!注意Constant里的这个data要是const类型的，总之就是Constant的data一旦初始化后没办法被修改
-        dtype_dict = {'int': np.int, 'float': np.float, 'int8': np.int8, 'int16': np.int16, 'int32': np.int32,
-                      'int64': np.int64, 'float32': np.float32, 'float64': np.float64}
-        data = np.asarray(data, dtype=dtype_dict[dtype])
+        dtype_dict = {'int': GlobalGraph.np.int, 'float': GlobalGraph.np.float, 'int8': GlobalGraph.np.int8, 'int16': GlobalGraph.np.int16, 'int32': GlobalGraph.np.int32,
+                      'int64': GlobalGraph.np.int64, 'float32': GlobalGraph.np.float32, 'float64': GlobalGraph.np.float64}
+        data = GlobalGraph.np.asarray(data, dtype=dtype_dict[dtype])
         # 显示调用父类的初始化函数
         Node.__init__(self,
                       in_bounds=in_bounds,
@@ -386,7 +414,7 @@ class Layer:
         # 如果传入的variables不是None，就赋值，否则初始化为空的数组
         self.variables = variables if variables else []
 
-    def __call__(self, inbound):
+    def __call__(self, inbound, *args, **kwargs):
         # 实际inbound可能是Layer也可能是Variable，需要重载，这里在父类里只实现了是Layer的情况，是Variable的情况在子类里在特殊实现。每个子类都会手动调用父类的__call__方法
         # 输入是Layer
         self.shape = self.compute_output_shape(inbound.shape)
@@ -426,19 +454,15 @@ class Layer:
                 total_params += v.data.size
         return total_params
 
-    def forward(self, x: Variable = None, is_training: bool = True, *args) -> Variable:
+    def forward(self, x: Variable = None, *args) -> Variable:
         # x代表本层的输入variable
         raise NotImplemented
 
     # 做一些forward后的初始化工作，每个子类forward后都会调用该函数
-    def connect_init(self, data: Variable, is_training: bool = True):
+    def feed_variable_to_next_layers(self, data: Variable):
         for out_bound in self.out_bounds:
             out_bound.input_data = data
 
-        if self.data.requires_grad and is_training:
-            initialize_ops_grad(self.data)
-            initialize_ops_grad(*self.variables)
-
     def backward(self, gradients: GlobalGraph.np.ndarray = None):
         for inbound in self.in_bounds:
             if inbound.data.requires_grad:
@@ -512,7 +536,7 @@ def mul(x: Variable, y: Variable) -> Variable:
                 break
         if matmul_flag:
             # 矩阵乘
-            outputs = Variable(in_bounds=[x, y], data=np.dot(x.data, y.data), requires_grad=x.requires_grad or y.requires_grad)
+            outputs = Variable(in_bounds=[x, y], data=GlobalGraph.np.dot(x.data, y.data), requires_grad=x.requires_grad or y.requires_grad)
             outputs.grad_fn = MatmulBackward
         else:
             # 点乘
@@ -528,8 +552,8 @@ def mul(x: Variable, y: Variable) -> Variable:
 
 
 def slices(x, item):
-    if GlobalGraph.inputs is None:
-        GlobalGraph.inputs = x
+    if GlobalGraph.INPUTS is None:
+        GlobalGraph.INPUTS = x
     ret = Variable(data=x.data[item], in_bounds=[x, ], requires_grad=x.requires_grad)
     if ret.requires_grad:
         x.out_bounds.append(ret)
